#!/bin/sh
lupdate -noobsolete ../mainform.cpp ../mainform.ui ../popplerdialog.ui ../configdialog.ui ../langselectdialog.ui ../main.cpp ../spellchecker.cpp ../sidebar.cpp ../settings.cpp ../configdialog.cpp ../langselectdialog.cpp ../recognitiondialog.ui ../recognitiondialog.cpp -ts yagf_ru.ts yagf_fr.ts yagf_uk.ts
