/*
    YAGF - cuneiform and tesseract OCR graphical front-end
    Copyright (C) 2009-2014 Andrei Borovsky <anb@symmetrica.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LANGSELECTDIALOG_H
#define LANGSELECTDIALOG_H

#include <QDialog>
#include <QStringList>
#include <QList>
#include <QListWidgetItem>

namespace Ui
{
class LangSelectDialog;
}

class LangSelectDialog : public QDialog
{
    Q_OBJECT

public:
    explicit LangSelectDialog(QWidget *parent = 0);
    ~LangSelectDialog();
protected:
    void accept();
private slots:
    void onItemClicked(QListWidgetItem *item);
private:
    QStringList getRecognitionLanguages() const;
    void fillLangs();
private:

    Ui::LangSelectDialog *ui;
    QList<QListWidgetItem *> items;
};

#endif // LANGSELECTDIALOG_H
