/*
    YAGF - cuneiform and tesseract OCR graphical front-ends
    Copyright (C) 2009-2014 Andrei Borovsky <anb@symmetrica.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef TIFFIMPORTER_H
#define TIFFIMPORTER_H

#include <QObject>
#include <QString>
#include <QStringList>

class TiffImporter : public QObject
{
    Q_OBJECT
public:
    explicit TiffImporter(const QString &fileName, QObject *parent = 0);
    void exec();
    QStringList extractedFiles();
signals:
    void error();
    void finished(const QStringList &files);
public slots:
private:
    QStringList files;
    QString tiffName;

};

#endif // TIFFIMPORTER_H
