/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/


#ifndef QXTUNIXSCINTERNAL_H
#define QXTUNIXSCINTERNAL_H

#include <QObject>

class QXtUnixSignalCatcherInternal  : public QObject
{
    Q_OBJECT

public:
    QXtUnixSignalCatcherInternal(QObject *parent = 0) :
        QObject(parent) {}
    void emitSignal(int sig_num) {
        emit unixSignalInternal(sig_num);
    }

signals:
    void unixSignalInternal(int sig_num);

};

#endif // QXTUNIXSCINTERNAL_H
