#ifndef SCANIMAGE_H
#define SCANIMAGE_H

#include <QObject>

class ScanImage : public QObject
{
    Q_OBJECT
public:
    explicit ScanImage(QObject *parent = 0);

signals:

public slots:

};

#endif // SCANIMAGE_H
