/*
   YAGF - cuneiform and tesseract OCR graphical front-end
   Copyright (C) 2009-2014 Andrei Borovsky <anb@symmetrica.net>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SUBIMAGEPP_H
#define SUBIMAGEPP_H

#include <QImage>
#include "ccbuilder.h"
#include "analysis.h"

class SubimagePP
{
public:
    explicit SubimagePP(QImage &image);
    ~SubimagePP();
    void fillComponents();
    void removeBars();
    void removeNoise();
    void stregthen();
signals:
    
public slots:
private:
    void labelsToBackground(QSet<quint32> &labels);
private:
    QImage &img;
    CCBuilder ccbuilder;
    QList<Rect> components;
    quint32 color;
    Bars bars;
    int mgw, mgh;
    quint32 darkestBG;
};

#endif // SUBIMAGEPP_H
