#ifndef SUBPAGEPP_H
#define SUBPAGEPP_H

#include <QObject>

class SubpagePP : public QObject
{
    Q_OBJECT
public:
    explicit SubpagePP(QObject *parent = 0);
    
signals:
    
public slots:
    
};

#endif // SUBPAGEPP_H
